==> cms_content_sync-8.x-1.33.tar.gz: cms_content_sync/src/Controller/ManualPull.php <==
<?php

namespace Drupal\cms_content_sync\Controller;

use Drupal\cms_content_sync\SyncCore\Query\Condition\DataCondition;
use Drupal\cms_content_sync\SyncCore\Query\Condition\ParentCondition;
use Drupal\cms_content_sync\SyncCore\Storage\ApiStorage;
use Drupal\cms_content_sync\SyncCore\Storage\EntityTypeStorage;
use Drupal\cms_content_sync\SyncCore\Storage\PreviewEntityStorage;
use Drupal\cms_content_sync\SyncIntent;
use Drupal\Core\Controller\ControllerBase;
use Drupal\cms_content_sync\Entity\Flow;
use Drupal\cms_content_sync\Entity\Pool;
use Drupal\cms_content_sync\PullIntent;
use Drupal\Core\Access\AccessResult;

/**
 * Provides a listing of Flow.
 */
class ManualPull extends ControllerBase {

  /**
   * Ensure that the pull tab is just show if a flow exists which contains
   * and entity type that has its pull set to "manual".
   */
  public function access() {
    $flows = Flow::getAll();
    $manually_pulled_entity_types = [];
    foreach ($flows as $flow) {
      if (!empty($flow->getEntityTypesToPull(PullIntent::PULL_MANUALLY))) {
        $manually_pulled_entity_types[$flow->id()] = $flow->getEntityTypesToPull(PullIntent::PULL_MANUALLY);
      }
    }

    return AccessResult::allowedIf(!empty($manually_pulled_entity_types))
      ->addCacheableDependency($flows);
  }

  /**
   * Render the content synchronization Angular frontend.
   *
   * @return array
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\cms_content_sync\SyncCore\Exception\SyncCoreException
   */
  public function content() {
    $settings = ContentSyncSettings::getInstance();

    $config = [
      'siteUrl' => $settings->getSiteBaseUrl(),
      'pools' => [],
      'flows' => [],
      'api_version' => ApiStorage::CUSTOM_API_VERSION,
      'entity_types' => [],
    ];

    $pools = Pool::getAll();

    $connection_id = NULL;
    foreach (Flow::getAll() as $id => $flow) {
      $config['flows'][$flow->id] = [
        'id' => $flow->id,
        'name' => $flow->name,
      ];

      foreach ($flow->getEntityTypeConfig() as $definition) {
        if (!$flow->canPullEntity($definition['entity_type_name'], $definition['bundle_name'], PullIntent::PULL_MANUALLY)) {
          continue;
        }

        foreach ($flow->getEntityTypeConfig($definition['entity_type_name'], $definition['bundle_name'])['import_pools'] as $id => $option) {
          if ($option != Pool::POOL_USAGE_ALLOW) {
            continue;
          }
          $pool = $pools[$id];
          $config['pools'][$pool->id] = [
            'id' => $pool->id,
            'label' => $pool->label,
            'site_id' => $pool->getSiteId(),
          ];

          if (empty($config['syncCoreUrl']) && _cms_content_sync_is_cloud_version() && $settings->isDirectSyncCoreAccessEnabled()) {
            $time = time();
            $authentication = AuthenticationByUser::getInstance();
            $hash = md5($time . ':' . $authentication->getPassword());
            $token = base64_encode(
              json_encode([
                't' => $time,
                'h' => $hash,
                's' => ContentSyncSettings::getInstance()->getSiteId(),
              ])
            );
            $config['syncCoreUrl'] = $pool->getBackendUrl();
            $config['syncCoreToken'] = $token;
          }
        }

        $index = $definition['entity_type_name'] . '.' . $definition['bundle_name'];
        if (!isset($config['entity_types'][$index])) {

          // Get the entity type and bundle name.
          $entity_type_storage = \Drupal::entityTypeManager()->getStorage($definition['entity_type_name']);
          $entity_type = $entity_type_storage->getEntityType();
          $entity_type_label = $entity_type->getLabel()->render();
          $bundle_info = \Drupal::service('entity_type.bundle.info')->getBundleInfo($definition['entity_type_name']);
          $bundle_label = $bundle_info[$definition['bundle_name']]['label'];

          $config['entity_types'][$index] = [
            'entity_type_name' => $definition['entity_type_name'],
            'entity_type_label' => $entity_type_label,
            'bundle_name' => $definition['bundle_name'],
            'bundle_label' => $bundle_label,
            'version' => $definition['version'],
            'pools' => [],
            'preview' => $definition['preview'] ?? Flow::PREVIEW_DISABLED,
          ];
        }
        else {
          if ($config['entity_types'][$index]['preview'] == Flow::PREVIEW_DISABLED || $definition['preview'] != Flow::PREVIEW_TABLE) {
            $config['entity_types'][$index]['preview'] = $definition['preview'];
          }
        }

        foreach ($definition['import_pools'] as $id => $action) {
          if (!isset($config['entity_types'][$index]['pools'][$id]) ||
            $action == Pool::POOL_USAGE_FORCE ||
            $config['entity_types'][$index]['pools'][$id] == Pool::POOL_USAGE_FORBID) {
            $config['entity_types'][$index]['pools'][$id] = $action;
          }
        }
      }
    }

    // Provide additional conditions for "subscribe only to" filters.
    if (!empty($config['syncCoreUrl'])) {
      $entity_type_ids = [];

      $filters = [];

      /**
       * @var \Drupal\Core\Entity\EntityFieldManager $entityFieldManager
       */
      $entityFieldManager = \Drupal::service('entity_field.manager');

      foreach (Flow::getAll() as $flow) {
        foreach ($flow->getEntityTypeConfig() as $definition) {
          if (!$flow->canPullEntity($definition['entity_type_name'], $definition['bundle_name'], PullIntent::PULL_MANUALLY)) {
            continue;
          }

          foreach ($definition['import_pools'] as $pool_id => $behavior) {
            if ($behavior != Pool::POOL_USAGE_ALLOW) {
              continue;
            }

            $id = EntityTypeStorage::getExternalEntityTypeId(
              $pool_id,
              $definition['entity_type_name'],
              $definition['bundle_name']
            );
            if (in_array($id, $entity_type_ids)) {
              continue;
            }

            /**
             * @var \Drupal\Core\Field\FieldDefinitionInterface[] $fields
             */
            $fields = $entityFieldManager->getFieldDefinitions($definition['entity_type_name'], $definition['bundle_name']);
            foreach ($fields as $key => $field) {
              $field_config = $flow->getFieldHandlerConfig($definition['entity_type_name'], $definition['bundle_name'], $key);
              if (empty($field_config)) {
                continue;
              }

              if (empty($field_config['handler_settings']['subscribe_only_to'])) {
                continue;
              }

              $allowed = [];

              foreach ($field_config['handler_settings']['subscribe_only_to'] as $ref) {
                $allowed[] = $ref['uuid'];
              }

              $filters[] = ParentCondition::any()
                ->add(
                  ParentCondition::none()
                    ->add(DataCondition::equal(PreviewEntityStorage::PROPERTY_ENTITY_TYPE_UNVERSIONED, $id))
                )
                ->add(DataCondition::in(PreviewEntityStorage::PROPERTY_CUSTOM_PROPERTIES . '.' . $key . '.' . SyncIntent::UUID_KEY, $allowed));
            }

            $entity_type_ids[] = $id;
          }
        }
      }

      if (count($filters) == 1) {
        $config['syncCoreCondition'] = $filters[0]->serialize();
      }
      elseif (count($filters) > 1) {
        $config['syncCoreCondition'] = ParentCondition::all($filters)->serialize();
      }
    }

    if (empty($config['entity_types'])) {
      \Drupal::messenger()->addMessage(t('There are no entity types to be pulled manually.'));
    }

    return [
      '#theme' => 'cms_content_sync_content_dashboard',
      '#configuration' => $config,
      '#attached' => ['library' => ['cms_content_sync/pull']],
    ];
  }

}

==> cms_content_sync-8.x-1.35.tar.gz: cms_content_sync/src/Controller/ManualPull.php <==
<?php

namespace Drupal\cms_content_sync\Controller;

use Drupal\cms_content_sync\Plugin\Type\EntityHandlerPluginManager;
use Drupal\cms_content_sync\SyncCoreInterface\DrupalApplication;
use Drupal\Core\Controller\ControllerBase;
use Drupal\cms_content_sync\Entity\Flow;
use Drupal\cms_content_sync\Entity\Pool;
use Drupal\cms_content_sync\PullIntent;
use Drupal\Core\Access\AccessResult;

/**
 * Provides a listing of Flow.
 */
class ManualPull extends ControllerBase {

  /**
   * Ensure that the pull tab is just show if a flow exists which contains
   * and entity type that has its pull set to "manual".
   */
  public function access() {
    $flows = Flow::getAll();
    $manually_pulled_entity_types = [];
    foreach ($flows as $flow) {
      if (!empty($flow->getEntityTypesToPull(PullIntent::PULL_MANUALLY))) {
        $manually_pulled_entity_types[$flow->id()] = $flow->getEntityTypesToPull(PullIntent::PULL_MANUALLY);
      }
    }

    return AccessResult::allowedIf(!empty($manually_pulled_entity_types))
      ->addCacheableDependency($flows);
  }

  /**
   * Render the content synchronization Angular frontend.
   *
   * @return array
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\cms_content_sync\SyncCore\Exception\SyncCoreException
   */
  public function content() {
    $settings = ContentSyncSettings::getInstance();

    $config = [
      'siteUrl' => $settings->getSiteBaseUrl(),
      'pools' => [],
      'flows' => [],
      'entity_types' => [],
    ];

    $pools = Pool::getAll();

    $cloud = _cms_content_sync_is_cloud_version() && $settings->isDirectSyncCoreAccessEnabled();
    $sync_core_settings = NULL;

    $connection_id = NULL;
    foreach (Flow::getAll() as $id => $flow) {
      $config['flows'][$flow->id] = [
        'id' => $flow->id,
        'name' => $flow->name,
      ];

      foreach ($flow->getEntityTypeConfig() as $definition) {
        if (!$flow->canPullEntity($definition['entity_type_name'], $definition['bundle_name'], PullIntent::PULL_MANUALLY)) {
          continue;
        }

        foreach ($flow->getEntityTypeConfig($definition['entity_type_name'], $definition['bundle_name'])['import_pools'] as $id => $option) {
          if ($option != Pool::POOL_USAGE_ALLOW) {
            continue;
          }
          $pool = $pools[$id];
          $config['pools'][$pool->id] = [
            'id' => $pool->id,
            'label' => $pool->label,
            'site_id' => DrupalApplication::get()->getSiteMachineName(),
          ];

          if (!$sync_core_settings) {
            $sync_core_settings = $pool
              ->getClient()
              ->getSyndicationService()
              ->configurePullDashboard();
          }
        }

        $index = $definition['entity_type_name'] . '.' . $definition['bundle_name'];
        if (!isset($config['entity_types'][$index])) {

          // Get the entity type and bundle name.
          $entity_type_storage = \Drupal::entityTypeManager()->getStorage($definition['entity_type_name']);
          $entity_type = $entity_type_storage->getEntityType();
          $entity_type_label = $entity_type->getLabel()->render();
          $bundle_info = \Drupal::service('entity_type.bundle.info')->getBundleInfo($definition['entity_type_name']);
          $bundle_label = $bundle_info[$definition['bundle_name']]['label'];

          $config['entity_types'][$index] = [
            'entity_type_name' => $definition['entity_type_name'],
            'entity_type_label' => $entity_type_label,
            'bundle_name' => $definition['bundle_name'],
            'bundle_label' => $bundle_label,
            'version' => $definition['version'],
            'pools' => [],
            'preview' => $definition['preview'] ?? Flow::PREVIEW_DISABLED,
          ];
        }
        else {
          if ($config['entity_types'][$index]['preview'] == Flow::PREVIEW_DISABLED || $definition['preview'] != Flow::PREVIEW_TABLE) {
            $config['entity_types'][$index]['preview'] = $definition['preview'];
          }
        }

        foreach ($definition['import_pools'] as $id => $action) {
          if (!isset($config['entity_types'][$index]['pools'][$id]) ||
            $action == Pool::POOL_USAGE_FORCE ||
            $config['entity_types'][$index]['pools'][$id] == Pool::POOL_USAGE_FORBID) {
            $config['entity_types'][$index]['pools'][$id] = $action;
          }
        }
      }
    }

    // Provide additional conditions for "subscribe only to" filters.
    if ($cloud) {
      $entity_type_ids = [];

      /**
       * @var \Drupal\Core\Entity\EntityFieldManager $entityFieldManager
       */
      $entityFieldManager = \Drupal::service('entity_field.manager');

      foreach (Flow::getAll() as $flow) {
        foreach ($flow->getEntityTypeConfig() as $definition) {
          if (!$flow->canPullEntity($definition['entity_type_name'], $definition['bundle_name'], PullIntent::PULL_MANUALLY)) {
            continue;
          }

          foreach ($definition['import_pools'] as $pool_id => $behavior) {
            if ($behavior != Pool::POOL_USAGE_ALLOW) {
              continue;
            }

            if (isset($entity_type_ids[$pool_id][$definition['entity_type_name']][$definition['bundle_name']])) {
              continue;
            }

            if (EntityHandlerPluginManager::isEntityTypeFieldable($definition['entity_type_name'])) {
              /**
               * @var \Drupal\Core\Field\FieldDefinitionInterface[] $fields
               */
              $fields = $entityFieldManager->getFieldDefinitions($definition['entity_type_name'], $definition['bundle_name']);
              foreach ($fields as $key => $field) {
                $field_config = $flow->getFieldHandlerConfig($definition['entity_type_name'], $definition['bundle_name'], $key);
                if (empty($field_config)) {
                  continue;
                }

                if (empty($field_config['handler_settings']['subscribe_only_to'])) {
                  continue;
                }

                $allowed = [];

                foreach ($field_config['handler_settings']['subscribe_only_to'] as $ref) {
                  $allowed[] = $ref['uuid'];
                }

                $sync_core_settings->ifTaggedWith(
                  $pool_id,
                  $definition['entity_type_name'],
                  $definition['bundle_name'],
                  $key,
                  $allowed
                );
              }
            }

            $entity_type_ids[$pool_id][$definition['entity_type_name']][$definition['bundle_name']] = TRUE;
          }
        }
      }
    }

    $config = array_merge($config, $sync_core_settings->getConfig());

    if (empty($config['entity_types'])) {
      \Drupal::messenger()->addMessage(t('There are no entity types to be pulled manually.'));
    }

    return [
      '#theme' => 'cms_content_sync_content_dashboard',
      '#configuration' => $config,
      '#attached' => ['library' => ['cms_content_sync/pull']],
    ];
  }

}
